@@ -451,17 +451,19 @@ class OAuthRemoteApp(object):
                       state (e.g. CSRF tokens).
         :param kwargs: add optional key/value pairs to the query string
         """
+        params = dict(self.request_token_params) or {}
+        params.update(**kwargs)
+
         if self.request_token_url:
             token = self.generate_request_token(callback)[0]
             url = '%s?oauth_token=%s' % (
                 self.expand_url(self.authorize_url), url_quote(token)
             )
+            if params:
+                url += '&' + url_encode(params)
         else:
             assert callback is not None, 'Callback is required OAuth2'
 
-            params = dict(self.request_token_params) or {}
-            params.update(**kwargs)
-
             client = self.make_client()
 
             if 'scope' in params:
@@ -548,6 +550,11 @@ class OAuthRemoteApp(object):
         client = self.make_client()
         client.verifier = request.args.get('oauth_verifier')
         tup = session.get('%s_oauthtok' % self.name)
+        if not tup:
+            raise OAuthException(
+                'Token not found, maybe you disabled cookie',
+                type='token_not_found'
+            )
         client.resource_owner_key = tup[0]
         client.resource_owner_secret = tup[1]
 
@@ -557,7 +564,8 @@ class OAuthRemoteApp(object):
         )
 
         resp, content = self.http_request(
-            uri, headers, to_bytes(data, self.encoding)
+            uri, headers, to_bytes(data, self.encoding),
+            method=self.access_token_method
         )
         data = parse_response(resp, content)
         if resp.code not in (200, 201):
