@@ -65,8 +65,8 @@ class OAuth2Provider(object):
 
         @app.route('/api/user')
         @oauth.require_oauth('email', 'username')
-        def user(oauth):
-            return jsonify(oauth.user)
+        def user():
+            return jsonify(request.oauth.user)
     """
 
     def __init__(self, app=None):
@@ -429,6 +429,9 @@ class OAuth2Provider(object):
                 for func in self._before_request_funcs:
                     func()
 
+                if hasattr(request, 'oauth') and request.oauth:
+                    return f(*args, **kwargs)
+
                 server = self.server
                 uri, http_method, body, headers = extract_params()
                 valid, req = server.verify_request(
@@ -439,8 +442,9 @@ class OAuth2Provider(object):
                     valid, req = func(valid, req)
 
                 if not valid:
-                    return abort(403)
-                return f(*((req,) + args), **kwargs)
+                    return abort(401)
+                request.oauth = req
+                return f(*args, **kwargs)
             return decorated
         return wrapper
 
@@ -464,7 +468,24 @@ class OAuth2RequestValidator(RequestValidator):
         self._grantsetter = grantsetter
 
     def client_authentication_required(self, request, *args, **kwargs):
-        return request.grant_type in ('password', 'refresh_token')
+        """Determine if client authentication is required for current request.
+
+        According to the rfc6749, client authentication is required in the
+        following cases:
+
+        Resource Owner Password Credentials Grant: see `Section 4.3.2`_.
+        Authorization Code Grant: see `Section 4.1.3`_.
+        Refresh Token Grant: see `Section 6`_.
+
+        .. _`Section 4.3.2`: http://tools.ietf.org/html/rfc6749#section-4.3.2
+        .. _`Section 4.1.3`: http://tools.ietf.org/html/rfc6749#section-4.1.3
+        .. _`Section 6`: http://tools.ietf.org/html/rfc6749#section-6
+        """
+        if request.grant_type == 'password':
+            return True
+        auth_required = ('authorization_code', 'refresh_token')
+        return 'Authorization' in request.headers and\
+                request.grant_type in auth_required
 
     def authenticate_client(self, request, *args, **kwargs):
         """Authenticate itself in other means.
@@ -494,6 +515,7 @@ class OAuth2RequestValidator(RequestValidator):
             return False
 
         request.client = client
+
         if client.client_secret != client_secret:
             log.debug('Authenticate client failed, secret not match.')
             return False
@@ -516,14 +538,12 @@ class OAuth2RequestValidator(RequestValidator):
             log.debug('Authenticate failed, client not found.')
             return False
 
+        if client.client_secret != request.client_secret:
+            log.debug('Authenticate client failed, secret not match.')
+            return False
+
         # attach client on request for convenience
         request.client = client
-
-        # authenticate non-confidential client_type only
-        # most of the clients are of public client_type
-        if client.client_type == 'confidential':
-            log.debug('Authenticate client failed, confidential client.')
-            return False
         return True
 
     def confirm_redirect_uri(self, client_id, code, redirect_uri, client,
@@ -535,9 +555,10 @@ class OAuth2RequestValidator(RequestValidator):
         add a `validate_redirect_uri` function on grant for a customized
         validation.
         """
+        client = client or self._clientgetter(client_id)
         log.debug('Confirm redirect uri for client %r and code %r.',
-                  client_id, code)
-        grant = self._grantgetter(client_id=client_id, code=code)
+                  client.client_id, code)
+        grant = self._grantgetter(client_id=client.client_id, code=code)
         if not grant:
             log.debug('Grant not found.')
             return False
@@ -674,10 +695,11 @@ class OAuth2RequestValidator(RequestValidator):
 
     def validate_code(self, client_id, code, client, request, *args, **kwargs):
         """Ensure the grant code is valid."""
+        client = client or self._clientgetter(client_id)
         log.debug(
-            'Validate code for client %r and code %r', client_id, code
+            'Validate code for client %r and code %r', client.client_id, code
         )
-        grant = self._grantgetter(client_id=client_id, code=code)
+        grant = self._grantgetter(client_id=client.client_id, code=code)
         if not grant:
             log.debug('Grant not found.')
             return False
@@ -707,19 +729,23 @@ class OAuth2RequestValidator(RequestValidator):
             log.debug('Password credential authorization is disabled.')
             return False
 
-        if grant_type not in ('authorization_code', 'password',
-                              'client_credentials', 'refresh_token'):
+        default_grant_types = (
+            'authorization_code', 'password',
+            'client_credentials', 'refresh_token',
+        )
+
+        if grant_type not in default_grant_types:
             return False
 
-        if hasattr(client, 'allowed_grant_types'):
-            return grant_type in client.allowed_grant_types
+        if hasattr(client, 'allowed_grant_types') and \
+           grant_type not in client.allowed_grant_types:
+            return False
 
         if grant_type == 'client_credentials':
-            if hasattr(client, 'user'):
-                request.user = client.user
-                return True
-            log.debug('Client should has a user property')
-            return False
+            if not hasattr(client, 'user'):
+                log.debug('Client should have a user property')
+                return False
+            request.user = client.user
 
         return True
 
